package cz.muni.fi.pa165.project.entity;

import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.UserRole;
import cz.muni.fi.pa165.project.repository.AppUserRepository;
import org.hibernate.exception.ConstraintViolationException;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;

import java.util.ArrayList;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

/**
 * @author Timotej Šulík | 485025
 */
class AppUserTest extends BaseEntityTest {
    @Autowired
    private AppUserRepository appUserRepository;

    private AppUser user;
    private AppUser otherUser;

    private AppUser sameEmailUser;
    private AppUser samePhoneUser;


    @BeforeEach
    void beforeMethod() {
        user = new AppUser("person@gmail.com", "123456ABCD", "John", "Smith", "+420720500600", Gender.MALE, UserRole.APP_USER);
        otherUser = new AppUser("other@gmail.com", "B", "Peter", "Tree", "+12720500600", Gender.MALE, UserRole.APP_USER);

        sameEmailUser = new AppUser("person@gmail.com", "9876WXYZ", "Peter", "Tree", "+421720500600", Gender.MALE, UserRole.APP_USER);
        samePhoneUser = new AppUser("other@gmail.com", "9876WXYZ", "Peter", "Tree", "+420720500600", Gender.MALE, UserRole.APP_USER);

        appUserRepository.save(user);
        appUserRepository.save(otherUser);
    }

    @Test
    void createAndPersistUserTest() {
        AppUser user = new AppUser("email", "password", "Name", "Surname", "+420123456789", Gender.MALE, UserRole.APP_USER);
        appUserRepository.save(user);
        assertThat(appUserRepository.findById(user.getId())).isPresent().get().isEqualTo(user);
    }

    @Test
    void findByIdTest() {
        assertThat(appUserRepository.findById(user.getId())).isPresent().get().isEqualTo(user);
    }

    @Test
    void existByIdTest() {
        assertThat(appUserRepository.existsById(user.getId())).isTrue();
    }

    @Test
    void findAllTest() {
        assertThat(appUserRepository.findAll()).containsExactly(user, otherUser);
    }

    @Test
    void findAllByIdTest() {
        List<Long> houseIdList = new ArrayList<>();
        houseIdList.add(user.getId());
        houseIdList.add(otherUser.getId());
        assertThat(appUserRepository.findAllById(houseIdList)).containsExactly(user, otherUser);
    }

    @Test
    void saveTest() {
        var tmpHouse = appUserRepository.findById(otherUser.getId());
        tmpHouse.orElseThrow().setGender(Gender.NON_BINARY);
        appUserRepository.save(tmpHouse.get());
        assertThat(appUserRepository.findById(tmpHouse.get().getId()).orElseThrow().getGender()).isEqualTo(Gender.NON_BINARY);

        assertThatThrownBy(() -> appUserRepository.save(sameEmailUser))
                .isInstanceOf(DataIntegrityViolationException.class)
                .hasCauseInstanceOf(ConstraintViolationException.class);
        assertThatThrownBy(() -> appUserRepository.save(samePhoneUser))
                .isInstanceOf(DataIntegrityViolationException.class)
                .hasCauseInstanceOf(ConstraintViolationException.class);
    }

    @Test
    void saveAllTest() {
        List<Long> houseIdList = new ArrayList<>();
        houseIdList.add(user.getId());
        houseIdList.add(otherUser.getId());

        var tmpHouses = appUserRepository.findAllById(houseIdList);
        assertThat(tmpHouses).containsExactly(user, otherUser);
        tmpHouses.forEach(h -> h.setGender(Gender.NON_BINARY));
        appUserRepository.saveAll(tmpHouses);
        assertThat(appUserRepository.findById(user.getId()).orElseThrow().getGender()).isEqualTo(Gender.NON_BINARY);
        assertThat(appUserRepository.findById(otherUser.getId()).orElseThrow().getGender()).isEqualTo(Gender.NON_BINARY);
    }

    @Test
    void countTest() {
        assertThat(appUserRepository.count()).isEqualTo(2);
    }

    @Test
    void deleteTest() {
        appUserRepository.delete(otherUser);
        assertThat(appUserRepository.existsById(otherUser.getId())).isFalse();
    }

    @Test
    void deleteByIdTest() {
        appUserRepository.deleteById(otherUser.getId());
        assertThat(appUserRepository.existsById(otherUser.getId())).isFalse();
    }

    @Test
    void deleteAllTest() {
        appUserRepository.deleteAll();
        assertThat(appUserRepository.findAll()).hasSize(0);
    }
}
