package cz.muni.fi.pa165.project.entity;

import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.UserRole;
import cz.muni.fi.pa165.project.repository.AppUserRepository;
import cz.muni.fi.pa165.project.repository.ConsumptionEventRepository;
import cz.muni.fi.pa165.project.repository.DeviceRepository;
import org.hibernate.exception.ConstraintViolationException;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;

import java.time.LocalDateTime;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

/**
 * @author Daniela Hostačná | 485432
 */
class ConsumptionEventTest extends BaseEntityTest {
    @Autowired
    private AppUserRepository appUserRepository;

    @Autowired
    private DeviceRepository deviceRepository;

    @Autowired
    private ConsumptionEventRepository consumptionEventRepository;

    private Device device;

    @BeforeEach
    void beforeMethod() {
        AppUser user = new AppUser("email1", "hash", "firstName", "lastName", "123456789", Gender.NON_BINARY, UserRole.APP_USER);
        appUserRepository.save(user);

        device = new Device(user, "Living room outlet", "Model XYZ 001", PowerStatus.ON);
        deviceRepository.save(device);
    }

    @Test
    void saveCorrectConsumptionEventTest() {
        ConsumptionEvent correctEvent = new ConsumptionEvent(device, LocalDateTime.now(), 25.3);

        assertThat(consumptionEventRepository.findAll()).hasSize(0);

        consumptionEventRepository.save(correctEvent);
        device.addConsumptionEvent(correctEvent);

        assertThat(consumptionEventRepository.findById(correctEvent.getId())).isPresent().get().isEqualTo(correctEvent);
        assertThat(deviceRepository.findById(device.getId()).orElseThrow().getConsumptionEvents()).containsExactly(correctEvent);
    }

    @Test
    void saveMissingDeviceConsumptionEventTest() {
        ConsumptionEvent missingDeviceConsumptionEvent = new ConsumptionEvent(null, LocalDateTime.now(), 25.3);

        assertThat(consumptionEventRepository.findAll()).hasSize(0);

        assertThatThrownBy(() -> consumptionEventRepository.save(missingDeviceConsumptionEvent))
                .isInstanceOf(DataIntegrityViolationException.class)
                .hasCauseInstanceOf(ConstraintViolationException.class);
    }

    @Test
    void saveMissingTimestampConsumptionEventTest() {
        ConsumptionEvent missingTimestampConsumptionEvent = new ConsumptionEvent(device, null, 25.3);

        assertThat(consumptionEventRepository.findAll()).hasSize(0);

        assertThatThrownBy(() -> consumptionEventRepository.save(missingTimestampConsumptionEvent))
                .isInstanceOf(DataIntegrityViolationException.class)
                .hasCauseInstanceOf(ConstraintViolationException.class);
    }

    @Test
    void updateConsumptionEventTest() {
        LocalDateTime time = LocalDateTime.now();

        ConsumptionEvent correctEvent1 = new ConsumptionEvent(device, time, 25.3);
        assertThat(consumptionEventRepository.findAll()).hasSize(0);
        consumptionEventRepository.save(correctEvent1);
        assertThat(consumptionEventRepository.findById(correctEvent1.getId()).orElseThrow()).isEqualTo(correctEvent1);
        correctEvent1.setTimestamp(time.plusSeconds(5));
        consumptionEventRepository.save(correctEvent1);
        assertThat(consumptionEventRepository.findById(correctEvent1.getId()).orElseThrow()).isEqualTo(correctEvent1);
    }

    @Test
    void deleteEntityTest() {
        ConsumptionEvent correctEvent = new ConsumptionEvent(device, LocalDateTime.now(), 25.3);
        assertThat(consumptionEventRepository.findAll()).hasSize(0);
        consumptionEventRepository.save(correctEvent);
        assertThat(consumptionEventRepository.findAll()).hasSize(1);
        consumptionEventRepository.delete(correctEvent);
        assertThat(consumptionEventRepository.findAll()).hasSize(0);
    }

    @Test
    void deleteByIdTest() {
        ConsumptionEvent correctEvent = new ConsumptionEvent(device, LocalDateTime.now(), 25.3);
        assertThat(consumptionEventRepository.findAll()).hasSize(0);
        consumptionEventRepository.save(correctEvent);
        assertThat(consumptionEventRepository.findAll()).hasSize(1);
        consumptionEventRepository.deleteById(correctEvent.getId());
        assertThat(consumptionEventRepository.findAll()).hasSize(0);
    }
}
