package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.DeviceDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.DeviceDetailDTO;
import cz.muni.fi.pa165.project.dto.DeviceUpdateDTO;
import cz.muni.fi.pa165.project.facade.DeviceFacade;
import cz.muni.fi.pa165.project.rest.ApiUris;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping(ApiUris.URI_DEVICES)
@Tag(name = "Devices", description = "Management of devices")
public class DeviceController {
    private static final Logger LOGGER = LoggerFactory.getLogger(DeviceController.class);
    private final DeviceFacade deviceFacade;

    @Autowired
    public DeviceController(DeviceFacade deviceFacade) {
        this.deviceFacade = deviceFacade;
    }

    @Operation(description = "Returns details about a device")
    @GetMapping("/{deviceId}")
    public final ResponseEntity<DeviceDetailDTO> getDetail(@PathVariable long deviceId) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(deviceFacade.getDevice(loggedInUsername, deviceId));
        } catch (Exception e) {
            LOGGER.error("Could not retrieve device information.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Updates a device")
    @PutMapping("/{deviceId}")
    public final ResponseEntity<Void> update(@PathVariable long deviceId, @RequestBody DeviceUpdateDTO device) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            deviceFacade.updateDevice(loggedInUsername, deviceId, device);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            LOGGER.error("Could not update the device.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Returns consumption events associated with the provided device")
    @GetMapping("/{deviceId}/consumption_events")
    public final ResponseEntity<List<DeviceDetailConsumptionEventDTO>> getConsumptionEvents(@PathVariable long deviceId, Pageable pageable) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(deviceFacade.getConsumptionEventsByDevice(loggedInUsername, deviceId, pageable).getContent());
        } catch (Exception e) {
            LOGGER.error("Could not retrieve consumption events for the device.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Returns count of all consumptions events associated with the provided device")
    @GetMapping("/{deviceId}/consumption_events/count")
    public final ResponseEntity<Long> getConsumptionEventsCount(@PathVariable long deviceId, Pageable pageable) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(deviceFacade.getConsumptionEventsByDevice(loggedInUsername, deviceId, pageable).getTotalElements());
        } catch (Exception e) {
            LOGGER.error("Could not retrieve the count of consumption events for the device.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Assigns a device to a house")
    @PostMapping("/{deviceId}/houses/{houseId}")
    public final ResponseEntity<Long> assignToHouse(@PathVariable long deviceId, @PathVariable long houseId) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(deviceFacade.assignDeviceToHouse(loggedInUsername, deviceId, houseId));
        } catch (Exception e) {
            LOGGER.error("Could not assign the device to the house.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Removes assignment from a device to a house")
    @DeleteMapping("/{deviceId}/houses/{houseId}")
    public final ResponseEntity<Void> unAssignFromHouse(@PathVariable long deviceId, @PathVariable long houseId) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            deviceFacade.unAssignDeviceFromHouse(loggedInUsername, deviceId, houseId);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            LOGGER.error("Could not unassign the device from the house.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Deletes a device")
    @DeleteMapping("/{deviceId}")
    public final ResponseEntity<Void> delete(@PathVariable long deviceId) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            deviceFacade.deleteDevice(loggedInUsername, deviceId);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            LOGGER.error("Could not delete the device.", e);
            return ResponseEntity.notFound().build();
        }
    }
}
