package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.HouseDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailDeviceDTO;
import cz.muni.fi.pa165.project.dto.HouseUpdateDetailDTO;
import cz.muni.fi.pa165.project.facade.HouseFacade;
import cz.muni.fi.pa165.project.rest.ApiUris;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping(ApiUris.URI_HOUSES)
@Tag(name = "Houses", description = "Management of houses")
public class HouseController {
    private static final Logger LOGGER = LoggerFactory.getLogger(HouseController.class);
    private final HouseFacade houseFacade;

    @Autowired
    public HouseController(HouseFacade houseFacade) {
        this.houseFacade = houseFacade;
    }

    @Operation(description = "Returns details about a single house")
    @GetMapping("/{houseId}")
    public final ResponseEntity<HouseDetailDTO> getDetail(@PathVariable long houseId) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(houseFacade.getHouseDetail(loggedInUsername, houseId));
        } catch (Exception e) {
            LOGGER.error("Could not retrieve house detail information.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Updates a house")
    @PutMapping("/{houseId}")
    public final ResponseEntity<Void> update(@PathVariable long houseId, @RequestBody HouseUpdateDetailDTO house) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            houseFacade.updateHouse(loggedInUsername, houseId, house);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            LOGGER.error("Could not update the house.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Deletes a house")
    @DeleteMapping("/{houseId}")
    public final ResponseEntity<Void> delete(@PathVariable long houseId) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            houseFacade.deleteHouse(loggedInUsername, houseId);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            LOGGER.error("Could not delete the house.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Shuts down power to a house")
    @PutMapping("/{houseId}/shut_down")
    public final ResponseEntity<Void> shutDown(@PathVariable long houseId) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            houseFacade.shutDown(loggedInUsername, houseId);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            LOGGER.error("Could not shut down power to the house.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Returns devices associated with the provided house")
    @GetMapping("/{houseId}/devices")
    public final ResponseEntity<List<HouseDetailDeviceDTO>> getDevices(@PathVariable long houseId, Pageable pageable) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(houseFacade.getDevicesByHouse(loggedInUsername, houseId, pageable).getContent());
        } catch (Exception e) {
            LOGGER.error("Could not retrieve devices for the house.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Returns count of all devices associated with the provided house")
    @GetMapping("/{houseId}/devices/count")
    public final ResponseEntity<Long> getDevicesCount(@PathVariable long houseId, Pageable pageable) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(houseFacade.getDevicesByHouse(loggedInUsername, houseId, pageable).getTotalElements());
        } catch (Exception e) {
            LOGGER.error("Could not retrieve the count of devices for the house.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Returns consumption events associated with the provided house")
    @GetMapping("/{houseId}/consumption_events")
    public final ResponseEntity<List<HouseDetailConsumptionEventDTO>> getConsumptionEvents(@PathVariable long houseId, Pageable pageable) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(houseFacade.getConsumptionEventsByHouse(loggedInUsername, houseId, pageable).getContent());
        } catch (Exception e) {
            LOGGER.error("Could not retrieve consumption events for the house.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Returns count of all consumption events associated with the provided house")
    @GetMapping("/{houseId}/consumption_events/count")
    public final ResponseEntity<Long> getConsumptionEventsCount(@PathVariable long houseId, Pageable pageable) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(houseFacade.getConsumptionEventsByHouse(loggedInUsername, houseId, pageable).getTotalElements());
        } catch (Exception e) {
            LOGGER.error("Could not retrieve the count of consumption events for the house.", e);
            return ResponseEntity.notFound().build();
        }
    }
}
