package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.UserChangePasswordDTO;
import cz.muni.fi.pa165.project.dto.UserProfileDTO;
import cz.muni.fi.pa165.project.dto.UserUpdateDTO;
import cz.muni.fi.pa165.project.facade.UserFacade;
import cz.muni.fi.pa165.project.rest.ApiUris;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(ApiUris.URI_USERS)
@Tag(name = "Users", description = "Management of users")
public class UserController {
    private static final Logger LOGGER = LoggerFactory.getLogger(UserController.class);
    private final UserFacade userFacade;

    @Autowired
    public UserController(UserFacade userFacade) {
        this.userFacade = userFacade;
    }

    @Operation(description = "Returns profile information of the current user")
    @GetMapping("/current/profile")
    public final ResponseEntity<UserProfileDTO> getCurrent() {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(userFacade.getUserByUserName(loggedInUsername));
        } catch (Exception e) {
            LOGGER.error("Could not retrieve user profile information.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Updates the user profile of the current user")
    @PutMapping("/current/profile")
    public final ResponseEntity<Void> updateCurrent(@RequestBody UserUpdateDTO user) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            userFacade.updateUser(loggedInUsername, user);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            LOGGER.error("Could not update user profile information.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Updates the user password of the current user")
    @PutMapping("/current/password")
    public final ResponseEntity<Void> changePassword(@RequestBody UserChangePasswordDTO user) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            userFacade.changePassword(loggedInUsername, user);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            LOGGER.error("Could not update user password.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Deletes the current user")
    @DeleteMapping("/current")
    public final ResponseEntity<Void> delete() {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            userFacade.deleteUser(loggedInUsername);
            return ResponseEntity.ok().build();
        } catch (Exception e) {
            LOGGER.error("Could not delete user.", e);
            return ResponseEntity.notFound().build();
        }
    }
}
