package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.HouseDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailConsumptionEventListDeviceDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailDeviceDTO;
import cz.muni.fi.pa165.project.dto.HouseUpdateDetailDTO;
import cz.muni.fi.pa165.project.enums.PowerStatusDto;
import cz.muni.fi.pa165.project.facade.HouseFacade;
import org.junit.jupiter.api.Test;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;

import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class HouseControllerTest extends BaseControllerTest<HouseFacade, HouseController> {

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void getHouseDetail() throws Exception {
        HouseDetailDTO house = new HouseDetailDTO(10, "name", "street", "city", "61200", "CZ");

        when(facade.getHouseDetail("user@mail.com", 10)).thenReturn(house);

        mockMvc.perform(get("/houses/10"))
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string(JsonStringMatcher.of(house)));
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void updateHouse() throws Exception {
        var device = new HouseUpdateDetailDTO("name", "street", "city", "61200", "CZ");

        var request = put("/houses/15")
                .contentType(MediaType.APPLICATION_JSON)
                .content(TestJsonSerializer.toJson(device));
        mockMvc.perform(request)
                .andExpect(status().isOk());

        verify(facade, times(1)).updateHouse("user@mail.com", 15, device);
        verifyNoMoreInteractions(facade);
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void deleteHouse() throws Exception {
        var request = delete("/houses/20");
        mockMvc.perform(request)
                .andExpect(status().isOk());

        verify(facade, times(1)).deleteHouse("user@mail.com", 20);
        verifyNoMoreInteractions(facade);
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void shutDown() throws Exception {
        var request = put("/houses/25/shut_down");
        mockMvc.perform(request)
                .andExpect(status().isOk());

        verify(facade, times(1)).shutDown("user@mail.com", 25);
        verifyNoMoreInteractions(facade);
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void getDevices() throws Exception {
        var devices = createDevices();

        when(facade.getDevicesByHouse("user@mail.com", 10, PageRequest.of(0, 2))).thenReturn(new PageImpl<>(devices));

        mockMvc.perform(get("/houses/10/devices?page=0&size=2"))
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string(JsonStringMatcher.of(devices)));
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void getConsumptionEvents() throws Exception {
        var events = createConsumptionEvents();

        when(facade.getConsumptionEventsByHouse("user@mail.com", 10, PageRequest.of(0, 2))).thenReturn(new PageImpl<>(events));

        mockMvc.perform(get("/houses/10/consumption_events?page=0&size=2"))
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string(JsonStringMatcher.of(events)));
    }

    private List<HouseDetailDeviceDTO> createDevices() {
        HouseDetailDeviceDTO deviceOne = new HouseDetailDeviceDTO(0, "a", "b", PowerStatusDto.ON);
        HouseDetailDeviceDTO deviceTwo = new HouseDetailDeviceDTO(0, "a", "b", PowerStatusDto.ON);
        return Arrays.asList(deviceOne, deviceTwo);
    }

    private List<HouseDetailConsumptionEventDTO> createConsumptionEvents() {
        var deviceOne = new HouseDetailConsumptionEventListDeviceDTO(123, "deva");
        var eventOne = new HouseDetailConsumptionEventDTO(486, LocalDateTime.of(2021, 12, 12, 1, 1), 2.5, deviceOne);
        var deviceTwo = new HouseDetailConsumptionEventListDeviceDTO(1489, "devb");
        var eventTwo = new HouseDetailConsumptionEventDTO(10, LocalDateTime.of(2022, 12, 12, 1, 2), 20, deviceTwo);
        return Arrays.asList(eventOne, eventTwo);
    }
}
