package cz.muni.fi.pa165.project.facade;

import cz.muni.fi.pa165.project.config.DeviceCreateMapper;
import cz.muni.fi.pa165.project.config.DeviceDetailConsumptionEventMapper;
import cz.muni.fi.pa165.project.config.DeviceDetailMapper;
import cz.muni.fi.pa165.project.config.DeviceListMapper;
import cz.muni.fi.pa165.project.config.DeviceUpdateMapper;
import cz.muni.fi.pa165.project.dto.DeviceAssignHouseDTO;
import cz.muni.fi.pa165.project.dto.DeviceCreateDTO;
import cz.muni.fi.pa165.project.dto.DeviceDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.DeviceDetailDTO;
import cz.muni.fi.pa165.project.dto.DeviceListDTO;
import cz.muni.fi.pa165.project.dto.DeviceUpdateDTO;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.service.AppUserService;
import cz.muni.fi.pa165.project.service.ConsumptionEventService;
import cz.muni.fi.pa165.project.service.DeviceService;
import cz.muni.fi.pa165.project.service.HouseDeviceService;
import cz.muni.fi.pa165.project.service.HouseService;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * @author Daniela Hostačná | 485432
 */
@Service
@Transactional
public class DeviceFacadeImpl implements DeviceFacade {
    private final DeviceService deviceService;
    private final AppUserService appUserService;
    private final ConsumptionEventService consumptionEventService;
    private final HouseService houseService;
    private final HouseDeviceService houseDeviceService;
    private final DeviceListMapper deviceListMapper;
    private final DeviceUpdateMapper deviceUpdateMapper;
    private final DeviceDetailMapper deviceDetailMapper;
    private final DeviceCreateMapper deviceCreateMapper;
    private final DeviceDetailConsumptionEventMapper deviceDetailConsumptionEventMapper;

    public DeviceFacadeImpl(DeviceService deviceService, AppUserService appUserService, ConsumptionEventService consumptionEventService, HouseService houseService, HouseDeviceService houseDeviceService, DeviceListMapper deviceListMapper,DeviceUpdateMapper deviceUpdateMapper, DeviceDetailMapper deviceDetailMapper, DeviceCreateMapper deviceCreateMapper, DeviceDetailConsumptionEventMapper deviceDetailConsumptionEventMapper) {
        this.deviceService = deviceService;
        this.appUserService = appUserService;
        this.consumptionEventService = consumptionEventService;
        this.houseService = houseService;
        this.houseDeviceService = houseDeviceService;
        this.deviceListMapper = deviceListMapper;
        this.deviceUpdateMapper = deviceUpdateMapper;
        this.deviceDetailMapper = deviceDetailMapper;
        this.deviceCreateMapper = deviceCreateMapper;
        this.deviceDetailConsumptionEventMapper = deviceDetailConsumptionEventMapper;
    }

    @Override
    public Page<DeviceListDTO> getDevicesByUser(String loggedInUsername, Pageable pageable) {
        var user = appUserService.findByEmail(loggedInUsername);

        if (user == null) {
            throw new IllegalArgumentException("Requester user does not exist");
        }

        return deviceListMapper.map(deviceService.findDevicesForUser(user, pageable));
    }

    @Override
    public DeviceDetailDTO getDevice(String loggedInUsername, long deviceId) {
        var user = appUserService.findByEmail(loggedInUsername);
        var device = deviceService.findById(deviceId);
        validateDeviceOwnership(user, device);

        return deviceDetailMapper.map(device);
    }

    @Override
    public void updateDevice(String loggedInUsername, long deviceId, DeviceUpdateDTO updateDeviceDto) {
        var user = appUserService.findByEmail(loggedInUsername);
        var device = deviceService.findById(deviceId);
        validateDeviceOwnership(user, device);

        var newDevice = deviceUpdateMapper.map(updateDeviceDto);
        newDevice.setId(deviceId);

        deviceService.update(newDevice);
    }

    @Override
    public Long createDevice(String loggedInUsername, DeviceCreateDTO deviceCreateDTO) {
        var user = appUserService.findByEmail(loggedInUsername);

        var device = deviceCreateMapper.map(deviceCreateDTO);
        device.setOwner(user);
        return deviceService.create(device);
    }

    @Override
    public Page<DeviceDetailConsumptionEventDTO> getConsumptionEventsByDevice(String loggedInUsername, long deviceId, Pageable pageable) {
        var user = appUserService.findByEmail(loggedInUsername);
        var device = deviceService.findById(deviceId);
        validateDeviceOwnership(user, device);

        return deviceDetailConsumptionEventMapper.map(consumptionEventService.findConsumptionEventsForDevice(device, pageable));
    }

    @Override
    public long assignDeviceToHouse(String loggedInUsername, long deviceId, long houseId) {
        var user = appUserService.findByEmail(loggedInUsername);
        var device = deviceService.findById(deviceId);
        var house = houseService.findById(houseId);
        validateDeviceOwnership(user, device);

        if (house == null) {
            throw new IllegalArgumentException("House does not exist");
        }
        if (house.getOwner() != user) {
            throw new IllegalArgumentException("The user does not own the house.");
        }
        if (device.getHouseDevices().stream().anyMatch(houseDevice -> houseDevice.getValidTo() == null)) {
            throw new IllegalArgumentException("Device is already assigned.");
        }

        var assignment = new HouseDevice();
        assignment.setHouse(house);
        assignment.setDevice(device);
        assignment.setValidFrom(LocalDateTime.now());

        return houseDeviceService.create(assignment);
    }

    @Override
    public void unAssignDeviceFromHouse(String loggedInUsername, long deviceId, long houseId) {
        var user = appUserService.findByEmail(loggedInUsername);
        var device = deviceService.findById(deviceId);
        var house = houseService.findById(houseId);
        validateDeviceOwnership(user, device);

        if (house == null) {
            throw new IllegalArgumentException("House does not exist");
        }
        if (house.getOwner() != user) {
            throw new IllegalArgumentException("The user does not own the house.");
        }

        houseDeviceService.closeAssignmentFor(device, house);
    }

    @Override
    public void deleteDevice(String loggedInUsername, long deviceId) {
        var user = appUserService.findByEmail(loggedInUsername);
        var device = deviceService.findById(deviceId);
        validateDeviceOwnership(user, device);

        deviceService.delete(device);
    }

    private void validateDeviceOwnership(AppUser user, Device device) {
        if (user == null) {
            throw new IllegalArgumentException("User does not exist.");
        }
        if (device == null) {
            throw new IllegalArgumentException("Device does not exist.");
        }
        if (!device.getOwner().equals(user)) {
            throw new IllegalArgumentException("The user is not owner of the device.");
        }
    }
}
