package cz.muni.fi.pa165.project.facade;

import cz.muni.fi.pa165.project.config.HouseCreateMapper;
import cz.muni.fi.pa165.project.config.HouseDetailConsumptionEventMapper;
import cz.muni.fi.pa165.project.config.HouseDetailDeviceMapper;
import cz.muni.fi.pa165.project.config.HouseDetailMapper;
import cz.muni.fi.pa165.project.config.HouseListMapper;
import cz.muni.fi.pa165.project.config.HouseUpdateDetailMapper;
import cz.muni.fi.pa165.project.dto.HouseCreateDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailDeviceDTO;
import cz.muni.fi.pa165.project.dto.HouseListDTO;
import cz.muni.fi.pa165.project.dto.HouseUpdateDetailDTO;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.BaseEntity;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.service.AppUserService;
import cz.muni.fi.pa165.project.service.ConsumptionEventService;
import cz.muni.fi.pa165.project.service.DeviceService;
import cz.muni.fi.pa165.project.service.HouseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.stream.Collectors;

/**
 * @author Daniela Hostačná | 485432
 */
@Service
@Transactional
public class HouseFacadeImpl implements HouseFacade {

    private final HouseService houseService;
    private final AppUserService appUserService;
    private final DeviceService deviceService;
    private final ConsumptionEventService consumptionEventService;

    private final HouseDetailMapper houseDetailMapper;
    private final HouseListMapper houseListMapper;
    private final HouseUpdateDetailMapper houseUpdateDetailMapper;
    private final HouseCreateMapper houseCreateMapper;
    private final HouseDetailDeviceMapper houseDetailDeviceMapper;
    private final HouseDetailConsumptionEventMapper houseDetailConsumptionEventMapper;

    @Autowired
    public HouseFacadeImpl(HouseService houseService, AppUserService appUserService, DeviceService deviceService, ConsumptionEventService consumptionEventService, HouseDetailMapper mapper, HouseListMapper houseListMapper, HouseUpdateDetailMapper houseUpdateDetailMapper, HouseCreateMapper houseCreateMapper, HouseDetailDeviceMapper houseDetailDeviceMapper, HouseDetailConsumptionEventMapper houseDetailConsumptionEventMapper) {
        this.houseService = houseService;
        this.appUserService = appUserService;
        this.deviceService = deviceService;
        this.consumptionEventService = consumptionEventService;
        this.houseDetailMapper = mapper;
        this.houseListMapper = houseListMapper;
        this.houseUpdateDetailMapper = houseUpdateDetailMapper;
        this.houseCreateMapper = houseCreateMapper;
        this.houseDetailDeviceMapper = houseDetailDeviceMapper;
        this.houseDetailConsumptionEventMapper = houseDetailConsumptionEventMapper;
    }

    @Override
    public Page<HouseListDTO> getHousesByUser(String loggedInUsername, Pageable pageable) {
        var user = appUserService.findByEmail(loggedInUsername);

        if (user == null) {
            throw new IllegalArgumentException("Requester user does not exist");
        }

        return houseListMapper.map(houseService.findHousesForUser(user, pageable));
    }

    @Override
    public HouseDetailDTO getHouseDetail(String loggedInUsername, long houseId) {
        var user = appUserService.findByEmail(loggedInUsername);
        var house = houseService.findById(houseId);
        validateHouseOwnership(user, house);

        return houseDetailMapper.map(house);
    }

    @Override
    public void updateHouse(String loggedInUsername, long houseId, HouseUpdateDetailDTO updateHouseDto) {
        var user = appUserService.findByEmail(loggedInUsername);
        var house = houseService.findById(houseId);
        validateHouseOwnership(user, house);

        var newHouse = houseUpdateDetailMapper.map(updateHouseDto);
        newHouse.setId(houseId);

        houseService.update(newHouse);
    }

    @Override
    public void deleteHouse(String loggedInUsername, long houseId) {
        var user = appUserService.findByEmail(loggedInUsername);
        var house = houseService.findById(houseId);
        validateHouseOwnership(user, house);

        houseService.delete(house);
    }

    @Override
    public void shutDown(String loggedInUsername, long houseId) {
        var user = appUserService.findByEmail(loggedInUsername);
        var house = houseService.findById(houseId);
        validateHouseOwnership(user, house);

        var ids = house.getHouseDevices()
                .stream()
                .map(houseDevice -> houseDevice.getDevice().getId())
                .collect(Collectors.toList());
        deviceService.shutDownAllDevicesByIds(ids);
    }

    @Override
    public long createHouse(String loggedInUsername, HouseCreateDTO houseCreateDTO) {
        var user = appUserService.findByEmail(loggedInUsername);

        var house = houseCreateMapper.map(houseCreateDTO);
        house.setOwner(user);
        return houseService.create(house);
    }

    @Override
    public Page<HouseDetailDeviceDTO> getDevicesByHouse(String loggedInUsername, long houseId, Pageable pageable) {
        var user = appUserService.findByEmail(loggedInUsername);
        var house = houseService.findById(houseId);
        validateHouseOwnership(user, house);

        var deviceIds = house.getHouseDevices().stream().map(HouseDevice::getDevice).map(BaseEntity::getId).collect(Collectors.toList());

        return houseDetailDeviceMapper.map(deviceService.findDevicesByIds(deviceIds, pageable));
    }

    @Override
    public Page<HouseDetailConsumptionEventDTO> getConsumptionEventsByHouse(String loggedInUsername, long houseId, Pageable pageable) {
        var user = appUserService.findByEmail(loggedInUsername);
        var house = houseService.findById(houseId);
        validateHouseOwnership(user, house);

        var deviceIds = house.getHouseDevices().stream().map(HouseDevice::getDevice).map(BaseEntity::getId).collect(Collectors.toList());

        return houseDetailConsumptionEventMapper.map(consumptionEventService.findConsumptionEventsForDevices(deviceIds, pageable));
    }

    private void validateHouseOwnership(AppUser user, House house) {
        if (user == null) {
            throw new IllegalArgumentException("User does not exist.");
        }
        if (house == null) {
            throw new IllegalArgumentException("House does not exist.");
        }
        if (!house.getOwner().equals(user)) {
            throw new IllegalArgumentException("The user is not owner of the house.");
        }
    }
}
