package cz.muni.fi.pa165.project.facade;

import cz.muni.fi.pa165.project.config.UserProfileMapper;
import cz.muni.fi.pa165.project.config.UserUpdateMapper;
import cz.muni.fi.pa165.project.dto.UserChangePasswordDTO;
import cz.muni.fi.pa165.project.dto.UserProfileDTO;
import cz.muni.fi.pa165.project.dto.UserUpdateDTO;
import cz.muni.fi.pa165.project.service.AppUserService;
import cz.muni.fi.pa165.project.service.DeviceService;
import cz.muni.fi.pa165.project.service.HouseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

/**
 * @author Daniela Hostačná | 485432
 */
@Service
@Transactional
public class UserFacadeImpl implements UserFacade {
    private final AppUserService appUserService;
    private final DeviceService deviceService;
    private final HouseService houseService;
    private final UserProfileMapper userProfileMapper;
    private final UserUpdateMapper userUpdateMapper;
    private final PasswordEncoder passwordEncoder;

    @Autowired
    public UserFacadeImpl(AppUserService appUserService, DeviceService deviceService, HouseService houseService, UserProfileMapper mapper, UserUpdateMapper userUpdateMapper, PasswordEncoder passwordEncoder) {
        this.appUserService = appUserService;
        this.deviceService = deviceService;
        this.houseService = houseService;
        this.userProfileMapper = mapper;
        this.userUpdateMapper = userUpdateMapper;
        this.passwordEncoder = passwordEncoder;
    }

    @Override
    public UserProfileDTO getUser(long userId) {
        return userProfileMapper.map(appUserService.findById(userId));
    }

    @Override
    public void updateUser(String loggedInUsername, UserUpdateDTO userUpdateDTO) {
        var user = appUserService.findByEmail(loggedInUsername);

        if (user == null) {
            throw new IllegalArgumentException("User attempting to make changes does not exist.");
        }
        if (userUpdateDTO == null) {
            throw new IllegalArgumentException("User to update cannot be null");
        }

        var userEntity = userUpdateMapper.map(userUpdateDTO);
        userEntity.setId(user.getId());
        userEntity.setEmail(user.getEmail());
        userEntity.setPasswordHash(user.getPasswordHash());
        userEntity.setRole(user.getRole());

        appUserService.update(userEntity);
    }

    @Override
    public void changePassword(String loggedInUsername, UserChangePasswordDTO password) {
        var user = appUserService.findByEmail(loggedInUsername);

        if (user == null) {
            throw new IllegalArgumentException("User attempting to make changes does not exist.");
        }

        if (!passwordEncoder.matches(password.getOldPassword(), user.getPasswordHash())) {
            throw new IllegalArgumentException("Incorrect old password. Change denied.");
        }
        if (!Objects.equals(password.getNewPassword(), password.getNewPasswordAgain())) {
            throw new IllegalArgumentException("New passwords do not match.");
        }

        String newPasswordHash = passwordEncoder.encode(password.getNewPassword());
        user.setPasswordHash(newPasswordHash);

        appUserService.update(user);
    }

    @Override
    public void deleteUser(String loggedInUsername) {
        var user = appUserService.findByEmail(loggedInUsername);

        deviceService.deleteAllForUser(user);
        houseService.deleteAllForUser(user);

        appUserService.delete(user);
    }

    @Override
    public UserProfileDTO getUserByUserName(String username) {
        return userProfileMapper.map(appUserService.findByEmail(username));
    }
}
