package cz.muni.fi.pa165.project.facade;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.config.HouseCreateMapper;
import cz.muni.fi.pa165.project.config.HouseDetailConsumptionEventMapper;
import cz.muni.fi.pa165.project.config.HouseDetailDeviceMapper;
import cz.muni.fi.pa165.project.config.HouseDetailMapper;
import cz.muni.fi.pa165.project.config.HouseListMapper;
import cz.muni.fi.pa165.project.config.HouseUpdateDetailMapper;
import cz.muni.fi.pa165.project.dto.HouseCreateDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailConsumptionEventListDeviceDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailDeviceDTO;
import cz.muni.fi.pa165.project.dto.HouseListDTO;
import cz.muni.fi.pa165.project.dto.HouseUpdateDetailDTO;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.ConsumptionEvent;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.PowerStatusDto;
import cz.muni.fi.pa165.project.enums.UserRole;
import cz.muni.fi.pa165.project.service.AppUserService;
import cz.muni.fi.pa165.project.service.ConsumptionEventService;
import cz.muni.fi.pa165.project.service.DeviceService;
import cz.muni.fi.pa165.project.service.HouseService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Set;

import static org.assertj.core.api.AssertionsForInterfaceTypes.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;

/**
 * @author Aneta Čadová | 485025
 */
class HouseFacadeTest extends BaseServiceLayerTest {
    @MockBean
    HouseService houseService;

    @MockBean
    AppUserService appUserService;

    @MockBean
    DeviceService deviceService;

    @MockBean
    ConsumptionEventService consumptionEventService;

    @MockBean
    HouseDetailMapper houseDetailMapper;

    @MockBean
    HouseListMapper houseListMapper;

    @MockBean
    HouseUpdateDetailMapper houseUpdateDetailMapper;

    @MockBean
    HouseCreateMapper houseCreateMapper;

    @MockBean
    HouseDetailDeviceMapper houseDetailDeviceMapper;

    @MockBean
    HouseDetailConsumptionEventMapper houseDetailConsumptionEventMapper;

    @Autowired
    private HouseFacadeImpl houseFacade;

    private final Pageable pageable = PageRequest.of(0, 2);
    private final AppUser user = new AppUser("email", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
    private final Device device = new Device(user, "nickName", "model", PowerStatus.ON);
    private final House house = new House(user, "FI MUNI", "Brno", "CZ", "Botanicka 68A", "60200");

    private final ConsumptionEvent consumptionEvent = new ConsumptionEvent(device, LocalDateTime.now(), 25.3);
    private final Page<ConsumptionEvent> consumptionEventPage = new PageImpl<>(List.of(consumptionEvent), pageable, 2);
    private final HouseDevice houseDevice = new HouseDevice(house, device, LocalDateTime.now().minusDays(2), LocalDateTime.now().minusDays(1));
    private final Page<House> housesPage = new PageImpl<>(List.of(house), pageable, 2);

    @BeforeEach
    void setup() {
        user.setId(1L);
        device.setId(1L);
        house.setId(1L);
        consumptionEvent.setId(1L);
        house.setHouseDevices(Set.of(houseDevice));
    }

    @Test
    void getHousesByUser() {
        HouseListDTO houseListDTO = new HouseListDTO(house.getId(), house.getName(), house.getStreet(), house.getCity(), house.getZip(), house.getCountry(), 1);
        Page<HouseListDTO> houseListDTOPage = new PageImpl<>(List.of(houseListDTO), pageable, 2);

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(houseService.findHousesForUser(user, PageRequest.of(0, 2))).thenReturn(housesPage);
        Mockito.when(houseListMapper.map(housesPage)).thenReturn(houseListDTOPage);

        var houseListDTOPageTest = houseFacade.getHousesByUser(user.getEmail(), PageRequest.of(0, 2));

        assertThat(houseListDTOPageTest).isNotNull();
        assertThat(houseListDTOPageTest).isEqualTo(houseListDTOPage);
    }

    @Test
    void getHousesByUserNullUserExceptionTest() {
        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(null);
        assertThrows(IllegalArgumentException.class, () -> houseFacade.getHousesByUser(user.getEmail(), PageRequest.of(0, 2)), "Requester user does not exist");
    }

    @Test
    void getHouseDetail() {
        HouseDetailDTO houseDetailDTO = new HouseDetailDTO(house.getId(), house.getName(), house.getStreet(), house.getCity(), house.getZip(), house.getCountry());

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(houseService.findById(house.getId())).thenReturn(house);
        Mockito.when(houseDetailMapper.map(house)).thenReturn(houseDetailDTO);

        var houseDetailDTOTest = houseFacade.getHouseDetail(user.getEmail(), house.getId());

        assertThat(houseDetailDTOTest).isNotNull();
        assertThat(houseDetailDTOTest).isEqualTo(houseDetailDTO);
    }

    @Test
    void updateHouse() {
        HouseUpdateDetailDTO houseUpdateDetailDTO = new HouseUpdateDetailDTO("updatedName", "updatedStreet", "updatedCity", "updatedZip", "updatedCountry");
        House updatedHouse = new House(null, houseUpdateDetailDTO.getName(), houseUpdateDetailDTO.getStreet(), houseUpdateDetailDTO.getCity(), houseUpdateDetailDTO.getZipCode(), houseUpdateDetailDTO.getCountry());

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(houseService.findById(house.getId())).thenReturn(house);
        Mockito.when(houseUpdateDetailMapper.map(houseUpdateDetailDTO)).thenReturn(updatedHouse);

        assertThat(updatedHouse.getId()).isNull();

        houseFacade.updateHouse(user.getEmail(), house.getId(), houseUpdateDetailDTO);

        Mockito.verify(houseService, times(1)).update(eq(updatedHouse));
        assertThat(updatedHouse.getId()).isEqualTo(house.getId());
    }

    @Test
    void deleteHouse() {
        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(houseService.findById(house.getId())).thenReturn(house);

        houseFacade.deleteHouse(user.getEmail(), house.getId());

        Mockito.verify(houseService, times(1)).delete(eq(house));
    }

    @Test
    void shutDown() {
        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(houseService.findById(house.getId())).thenReturn(house);

        houseFacade.shutDown(user.getEmail(), house.getId());
        Mockito.verify(deviceService, times(1)).shutDownAllDevicesByIds(eq(List.of(device.getId())));
    }

    @Test
    void createHouse() {
        House createdHouse = new House(null, "createName", "street", "city", "zip", "country");
        createdHouse.setId(2L);
        HouseCreateDTO houseCreateDTO = new HouseCreateDTO(createdHouse.getName(), createdHouse.getStreet(), createdHouse.getCity(), createdHouse.getZip(), createdHouse.getCountry());

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(houseCreateMapper.map(houseCreateDTO)).thenReturn(createdHouse);
        Mockito.when(houseService.create(createdHouse)).thenReturn(createdHouse.getId());
        Mockito.when(houseService.findById(createdHouse.getId())).thenReturn(createdHouse);

        var createdHouseId = houseFacade.createHouse(user.getEmail(), houseCreateDTO);

        assertThat(createdHouseId).isNotNull();
        assertThat(createdHouseId).isEqualTo(createdHouse.getId());
        assertThat(houseService.findById(createdHouseId).getOwner()).isEqualTo(user);
    }

    @Test
    void getDevicesByHouse() {
        Page<Device> devicePage = new PageImpl<>(List.of(device), pageable, 2);
        HouseDetailDeviceDTO houseDetailDeviceDTO = new HouseDetailDeviceDTO(device.getId(), device.getNickName(), device.getModel(), PowerStatusDto.ON);
        Page<HouseDetailDeviceDTO> houseDetailDeviceDTOPage = new PageImpl<>(List.of(houseDetailDeviceDTO), pageable, 2);

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(houseService.findById(house.getId())).thenReturn(house);
        Mockito.when(deviceService.findDevicesByIds(List.of(device.getId()), pageable)).thenReturn(devicePage);
        Mockito.when(houseDetailDeviceMapper.map(devicePage)).thenReturn(houseDetailDeviceDTOPage);

        var houseDetailDeviceDTOTest = houseFacade.getDevicesByHouse(user.getEmail(), house.getId(), PageRequest.of(0, 2));
        assertThat(houseDetailDeviceDTOTest).isNotNull();
        assertThat(houseDetailDeviceDTOTest).isEqualTo(houseDetailDeviceDTOPage);

    }

    @Test
    void getConsumptionEventsByHouse() {
        HouseDetailConsumptionEventListDeviceDTO houseDetailConsumptionEventListDeviceDTO = new HouseDetailConsumptionEventListDeviceDTO(device.getId(), device.getNickName());
        HouseDetailConsumptionEventDTO houseDetailConsumptionEventDTO = new HouseDetailConsumptionEventDTO(consumptionEvent.getId(), consumptionEvent.getTimestamp(), consumptionEvent.getPowerConsumption(), houseDetailConsumptionEventListDeviceDTO);
        Page<HouseDetailConsumptionEventDTO> houseDetailConsumptionEventDTOPage = new PageImpl<>(List.of(houseDetailConsumptionEventDTO), pageable, 2);

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(houseService.findById(house.getId())).thenReturn(house);
        Mockito.when(consumptionEventService.findConsumptionEventsForDevices(List.of(device.getId()), PageRequest.of(0, 2))).thenReturn(consumptionEventPage);
        Mockito.when(houseDetailConsumptionEventMapper.map(consumptionEventPage)).thenReturn(houseDetailConsumptionEventDTOPage);

        var houseDetailConsumptionEventDTOTest = houseFacade.getConsumptionEventsByHouse(user.getEmail(), house.getId(), PageRequest.of(0, 2));
        assertThat(houseDetailConsumptionEventDTOTest).isNotNull();
        assertThat(houseDetailConsumptionEventDTOTest).isEqualTo(houseDetailConsumptionEventDTOPage);
    }

}
