package cz.muni.fi.pa165.project.facade;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.config.UserProfileMapper;
import cz.muni.fi.pa165.project.config.UserUpdateMapper;
import cz.muni.fi.pa165.project.dto.UserChangePasswordDTO;
import cz.muni.fi.pa165.project.dto.UserProfileDTO;
import cz.muni.fi.pa165.project.dto.UserUpdateDTO;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.GenderDto;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.UserRole;
import cz.muni.fi.pa165.project.service.AppUserService;
import cz.muni.fi.pa165.project.service.DeviceService;
import cz.muni.fi.pa165.project.service.HouseService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.time.LocalDateTime;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;

/**
 * @author Aneta Čadová | 485025
 */
class UserFacadeTest extends BaseServiceLayerTest {
    @MockBean
    DeviceService deviceService;

    @MockBean
    AppUserService appUserService;

    @MockBean
    HouseService houseService;

    @MockBean
    UserProfileMapper userProfileMapper;

    @MockBean
    UserUpdateMapper userUpdateMapper;

    @Autowired
    private UserFacadeImpl userFacade;

    private final AppUser user1 = new AppUser("email1", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
    private final AppUser user2 = new AppUser("email2", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
    private final Device device = new Device(user1, "nickName", "model", PowerStatus.ON);
    private final House house = new House(user1, "FI MUNI", "Brno", "CZ", "Botanicka 68A", "60200");
    private final HouseDevice houseDevice = new HouseDevice(house, device, LocalDateTime.now().minusDays(2), LocalDateTime.now().minusDays(1));

    @BeforeEach
    void setup() {
        houseDevice.setId(1L);
        user1.setId(1L);
        user2.setId(2L);
        device.setId(1L);
        house.setId(1L);
    }

    @Test
    void getUserTest() {
        UserProfileDTO userProfileDTO = new UserProfileDTO(GenderDto.FEMALE, user1.getFirstName(), user1.getLastName(), user1.getPhoneNumber(), user1.getEmail());
        Mockito.when(appUserService.findById(user1.getId())).thenReturn(user1);
        Mockito.when(userProfileMapper.map(user1)).thenReturn(userProfileDTO);

        var userProfileDTOTest = userFacade.getUser(user1.getId());

        assertThat(userProfileDTOTest).isNotNull();
        assertThat(userProfileDTOTest).isEqualTo(userProfileDTO);
    }

    @Test
    void updateUserTest() {
        AppUser updatedUser = new AppUser(user1.getEmail(), user1.getPasswordHash(), "updatedFirstName", "updatedLastName", "updatedNumber", Gender.FEMALE, user1.getRole());
        UserUpdateDTO userUpdateDTO = new UserUpdateDTO(GenderDto.FEMALE, updatedUser.getFirstName(), updatedUser.getLastName(), updatedUser.getPhoneNumber());

        Mockito.when(appUserService.findByEmail(user1.getEmail())).thenReturn(user1);
        Mockito.when(userUpdateMapper.map(userUpdateDTO)).thenReturn(updatedUser);
        assertThat(updatedUser.getId()).isNull();

        userFacade.updateUser(user1.getEmail(), userUpdateDTO);

        Mockito.verify(appUserService, times(1)).update(eq(updatedUser));
        assertThat(updatedUser.getId()).isEqualTo(user1.getId());
    }

    @Test
    void updateUserNullUserExceptionTest() {
        Mockito.when(appUserService.findById(user1.getId())).thenReturn(null);
        assertThrows(IllegalArgumentException.class, () -> userFacade.updateUser(user1.getEmail(), new UserUpdateDTO()), "User attempting to make changes does not exist.");
    }

    @Test
    void updateUserNullUserProfileDtoExceptionTest() {
        Mockito.when(appUserService.findById(user1.getId())).thenReturn(user1);
        assertThrows(IllegalArgumentException.class, () -> userFacade.updateUser(user1.getEmail(), null), "User attempting to make changes does not exist.");
    }

    @Test
    void deleteUserTest() {
        Mockito.when(appUserService.findByEmail(user1.getEmail())).thenReturn(user1);

        userFacade.deleteUser(user1.getEmail());

        Mockito.verify(deviceService, times(1)).deleteAllForUser(eq(user1));
        Mockito.verify(houseService, times(1)).deleteAllForUser(eq(user1));
        Mockito.verify(appUserService, times(1)).delete(eq(user1));
    }

    @Test
    void changePasswordTest() {
        UserChangePasswordDTO userChangePasswordDTO = new UserChangePasswordDTO("hash", "newPassword", "newPassword");
        Mockito.when(appUserService.findByEmail(user1.getEmail())).thenReturn(user1);

        userFacade.changePassword(user1.getEmail(), userChangePasswordDTO);
        assertThat(user1.getPasswordHash()).isEqualTo(userChangePasswordDTO.getNewPassword());

        Mockito.verify(appUserService, times(1)).update(eq(user1));
    }

    @Test
    void changePasswordIncorrectOldPasswordExceptionTest() {
        UserChangePasswordDTO userChangePasswordDTO = new UserChangePasswordDTO("incorrectOldPassword", "newPassword", "newPassword");
        Mockito.when(appUserService.findByEmail(user1.getEmail())).thenReturn(user1);

        assertThrows(IllegalArgumentException.class, () -> userFacade.changePassword(user1.getEmail(), userChangePasswordDTO), "Incorrect old password. Change denied.");
    }

    @Test
    void changePasswordIncorrectNewPasswordAgainExceptionTest() {
        UserChangePasswordDTO userChangePasswordDTO = new UserChangePasswordDTO("hash", "newPassword", "newPasswordIncorrect");
        Mockito.when(appUserService.findById(user1.getId())).thenReturn(user1);

        assertThrows(IllegalArgumentException.class, () -> userFacade.changePassword(user1.getEmail(), userChangePasswordDTO), "New passwords do not match.");
    }

    @Test
    void changePasswordUserDoesntExistExceptionTest() {
        UserChangePasswordDTO userChangePasswordDTO = new UserChangePasswordDTO("hash", "newPassword", "newPassword");
        Mockito.when(appUserService.findById(user1.getId())).thenReturn(null);

        assertThrows(IllegalArgumentException.class, () -> userFacade.changePassword(user1.getEmail(), userChangePasswordDTO), "User attempting to make changes does not exist.");
    }
}
