package cz.muni.fi.pa165.project.service;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.repository.DeviceRepository;
import cz.muni.fi.pa165.project.repository.HouseDeviceRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;

/**
 * @author Aneta Čadová | 485025
 */
class DeviceServiceTest extends BaseServiceLayerTest {
    @MockBean
    private ConsumptionEventService consumptionEventService;

    @MockBean
    private DeviceRepository deviceRepository;

    @MockBean
    private HouseDeviceRepository houseDeviceRepository;

    @Autowired
    private DeviceServiceImpl deviceService;

    private final AppUser user = new AppUser();

    private final Device device1 = new Device(user, "nickName1", "model1", PowerStatus.ON);
    private final Device device2 = new Device(user, "nickName2", "model2", PowerStatus.ON);
    private final House house = new House(user, "FI MUNI", "Brno", "CZ", "Botanicka 68A", "60200");
    private final HouseDevice houseDevice = new HouseDevice(house, device1, LocalDateTime.now(), LocalDateTime.now().plusSeconds(1));


    private final List<Device> deviceList = new ArrayList<>();
    private final List<Long> deviceIdsList = new ArrayList<>();

    private final Pageable pageable = Mockito.mock(Pageable.class);
    private Page<Device> devicePage;

    @BeforeEach
    void setup() {
        device1.setId(1L);
        device2.setId(2L);
        houseDevice.setId(1L);

        user.setHouses(Set.of(house));
        user.setDevices(Set.of(device1));
        device1.setHouseDevices(Set.of(houseDevice));

        deviceList.add(device1);
        deviceList.add(device2);
        devicePage = new PageImpl<>(deviceList, pageable, 2);

        deviceIdsList.add(device1.getId());
        deviceIdsList.add(device2.getId());
    }

    @Test
    void deleteAllForUserTest() {
        deviceService.deleteAllForUser(user);

        Mockito.verify(deviceRepository, times(1)).deleteAllById(eq(List.of(1L)));
        Mockito.verify(consumptionEventService, times(1)).deleteAllByDeviceIds(eq(List.of(1L)));
        Mockito.verify(houseDeviceRepository, times(1)).deleteAllById(eq(List.of(houseDevice.getId())));
    }

    @Test
    void shutDownAllDevicesByIdsTest() {
        Mockito.when(deviceRepository.findAllById(deviceIdsList)).thenReturn(deviceList);
        deviceService.shutDownAllDevicesByIds(deviceIdsList);

        assertThat(device1.getPowerStatus()).isEqualTo(PowerStatus.OFF);
        assertThat(device2.getPowerStatus()).isEqualTo(PowerStatus.OFF);
    }

    @Test
    void findDevicesByIdsTest() {
        Mockito.when(deviceRepository.findAllByIds(deviceIdsList, pageable)).thenReturn(devicePage);

        Page<Device> devicesTestPage = deviceService.findDevicesByIds(deviceIdsList, pageable);
        var devicesTestList = devicesTestPage.get().collect(Collectors.toList());

        assertThat(devicesTestList).hasSize(2);
        assertThat(devicesTestList).containsExactly(device1, device2);
    }

    @Test
    void findDevicesForUserTest() {
        Mockito.when(deviceRepository.findAllByUser(user, pageable)).thenReturn(devicePage);

        Page<Device> devicesTestPage = deviceService.findDevicesForUser(user, pageable);
        var devicesTestList = devicesTestPage.get().collect(Collectors.toList());

        assertThat(devicesTestList).hasSize(2);
        assertThat(devicesTestList).containsExactly(device1, device2);
    }

    @Test
    void findByIdTest() {
        Mockito.when(deviceRepository.findById(1L)).thenReturn(java.util.Optional.of(device1));

        var testDevice = deviceService.findById(1);
        assertThat(testDevice).isNotNull();
        assertThat(testDevice).isEqualTo(device1);
    }

    @Test
    void createTest() {
        Mockito.when(deviceRepository.save(device1)).thenReturn(device1);
        var deviceId = deviceService.create(device1);

        assertThat(deviceId).isNotNull();
        assertThat(deviceId).isEqualTo(device1.getId());
    }

    @Test
    void deleteTest() {
        deviceService.delete(device1);
        Mockito.verify(deviceRepository, times(1)).delete(eq(device1));
        Mockito.verify(consumptionEventService, times(1)).deleteAllByDeviceIds(eq(List.of(device1.getId())));
        Mockito.verify(houseDeviceRepository, times(1)).deleteAllById(eq(List.of(houseDevice.getId())));
    }

}
