package cz.muni.fi.pa165.project.service;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.repository.HouseDeviceRepository;
import cz.muni.fi.pa165.project.repository.HouseRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;

/**
 * @author Aneta Čadová | 485025
 */
class HouseServiceTest extends BaseServiceLayerTest {
    @MockBean
    private HouseDeviceRepository houseDeviceRepository;

    @MockBean
    private HouseRepository houseRepository;

    @Autowired
    private HouseServiceImpl houseService;

    private final AppUser user = new AppUser();
    private final Device device = new Device(user, "nickName", "model", PowerStatus.ON);
    private final House house = new House(user, "FI MUNI", "Brno", "CZ", "Botanicka 68A", "60200");
    private final House house2 = new House(user, "name", "street", "city", "zip", "country");
    private final HouseDevice houseDevice = new HouseDevice(house, device, LocalDateTime.now().minusDays(1), LocalDateTime.now().minusHours(1));

    private final List<House> houseList = new ArrayList<>();

    private final Pageable pageable = Mockito.mock(Pageable.class);
    private Page<House> housePage;

    @BeforeEach
    void setup() {
        houseDevice.setId(1L);
        house.setId(1L);
        house2.setId(1L);

        user.setHouses(Set.of(house));
        house.setHouseDevices(Set.of(houseDevice));

        houseList.add(house);
        houseList.add(house2);

        housePage = new PageImpl<>(houseList, pageable, 2);
    }

    @Test
    void findByIdTest() {
        Mockito.when(houseRepository.findById(1L)).thenReturn(java.util.Optional.of(house));

        var testHouse = houseService.findById(1);
        assertThat(testHouse).isNotNull();
        assertThat(testHouse).isEqualTo(house);
    }

    @Test
    void create() {
        Mockito.when(houseRepository.save(house)).thenReturn(house);
        var houseId = houseService.create(house);

        assertThat(houseId).isNotNull();
        assertThat(houseId).isEqualTo(house.getId());
    }

    @Test
    void deleteAllForUserTest() {
        houseService.deleteAllForUser(user);

        Mockito.verify(houseDeviceRepository, times(1)).deleteAllById(eq(List.of(houseDevice.getId())));
        Mockito.verify(houseRepository, times(1)).deleteAllById(eq(List.of(house.getId())));
    }

    @Test
    void findHousesForUserTest() {
        Mockito.when(houseRepository.findAllByUser(user, pageable)).thenReturn(housePage);

        Page<House> houseTestPage = houseService.findHousesForUser(user, pageable);
        var houseTestList = houseTestPage.get().collect(Collectors.toList());

        assertThat(houseTestList).hasSize(2);
        assertThat(houseTestList).containsExactly(house, house2);
    }

    @Test
    void deleteTest() {
        houseService.delete(house);
        Mockito.verify(houseRepository, times(1)).delete(eq(house));
    }

    @Test
    void deleteByIdTest() {
        houseService.delete(house.getId());
        Mockito.verify(houseRepository, times(1)).deleteById(eq(house.getId()));
    }

    @Test
    void updateTest() {
        Mockito.when(houseRepository.findById(house2.getId())).thenReturn(java.util.Optional.of(house));
        houseService.update(house2);

        assertThat(house.getName()).isEqualTo(house2.getName());
        assertThat(house.getStreet()).isEqualTo(house2.getStreet());
        assertThat(house.getCity()).isEqualTo(house2.getCity());
        assertThat(house.getZip()).isEqualTo(house2.getZip());
        assertThat(house.getCountry()).isEqualTo(house2.getCountry());
    }
}
